@@ -5,7 +5,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2019	PgPool Global Development Group
+ * Copyright (c) 2003-2017	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -60,7 +60,8 @@
 #include "utils/elog.h"
 #include "auth/md5.h"
 #include "auth/pool_passwd.h"
-#include "utils/pool_relcache.h"
+#include "auth/pool_hba.h"
+
 
 static StartupPacket *read_startup_packet(POOL_CONNECTION *cp);
 static POOL_CONNECTION_POOL *connect_backend(StartupPacket *sp, POOL_CONNECTION *frontend);
@@ -92,17 +93,12 @@ static bool backend_cleanup(POOL_CONNECTION* volatile *frontend, POOL_CONNECTION
 static void free_persisten_db_connection_memory(POOL_CONNECTION_POOL_SLOT *cp);
 static int choose_db_node_id(char *str);
 static void child_will_go_down(int code, Datum arg);
-static int opt_sort(const void *a, const void *b);
 /*
- * Non 0 means SIGTERM (smart shutdown) or SIGINT (fast shutdown) has arrived
+ * non 0 means SIGTERM(smart shutdown) or SIGINT(fast shutdown) has arrived
  */
 volatile sig_atomic_t exit_request = 0;
 static volatile sig_atomic_t alarm_enabled = false;
 
-/*
- * Ignore SIGUSR1 if requested. Used when DROP DATABASE is requested.
- */
-volatile sig_atomic_t ignore_sigusr1 = 0;
 
 static int idle;		/* non 0 means this child is in idle state */
 static int accepted = 0;
@@ -138,14 +134,9 @@ void do_child(int *fds)
 	struct timezone tz;
 	struct timeval timeout;
 	static int connected = 0;	/* non 0 if has been accepted connections from frontend */
+	int connections_count = 0;	/* used if child_max_connections > 0 */
 	char psbuf[NI_MAXHOST + 128];
 
-	/* counter for child_max_connections.  "volatile" declaration is necessary
-	 * so that this is counted up even if long jump is issued due to
-	 * ereport(ERROR).
-	 */
-	volatile	int			connections_count = 0;
-
 	ereport(DEBUG2,
 			(errmsg("I am Pgpool Child process with pid: %d", getpid())));
 
@@ -245,10 +236,7 @@ void do_child(int *fds)
 		MemoryContextSwitchTo(ProcessLoopContext);
 
 		if (accepted)
-		{
-			accepted = 0;
 			connection_count_down();
-		}
         
         backend_cleanup(&child_frontend, backend, frontend_invalid);
 
@@ -297,8 +285,7 @@ void do_child(int *fds)
 		StartupPacket *sp;
 		int front_end_fd;
 		SockAddr saddr;
-
-		/* reset per iteration memory context */
+		/* rest per iteration memory context */
 		MemoryContextSwitchTo(ProcessLoopContext);
 		MemoryContextResetAndDeleteChildren(ProcessLoopContext);
 
@@ -523,11 +510,6 @@ static StartupPacket *read_startup_packet(POOL_CONNECTION *cp)
 	int protov;
 	int len;
 	char *p;
-	char **guc_options;
-	int opt_num = 0;
-	char *sp_sort;
-	char *tmpopt;
-	int i;
 
 	sp = (StartupPacket *)palloc0(sizeof(*sp));
 	enable_authentication_timeout();
@@ -554,6 +536,7 @@ static StartupPacket *read_startup_packet(POOL_CONNECTION *cp)
 	memcpy(&protov, sp->startup_packet, sizeof(protov));
 	sp->major = ntohl(protov)>>16;
 	sp->minor = ntohl(protov) & 0x0000ffff;
+	p = sp->startup_packet;
     cp->protoVersion = sp->major;
 
 	switch(sp->major)
@@ -570,46 +553,6 @@ static StartupPacket *read_startup_packet(POOL_CONNECTION *cp)
 			break;
 
 		case PROTO_MAJOR_V3: /* V3 */
-			/* copy startup_packet */
-			sp_sort = palloc0(len);
-			memcpy(sp_sort,sp->startup_packet,len);
-
-			p = sp_sort;
-			p += sizeof(int);	/* skip protocol version info */
-			/* count the number of options */
-			while(*p)
-			{
-				p += (strlen(p) + 1); /* skip option name */
-				p += (strlen(p) + 1); /* skip option value */
-				opt_num ++;
-			}
-			guc_options = (char **)palloc0(opt_num * sizeof(char *));
-			/* get guc_option name list */
-			p = sp_sort + sizeof(int);
-			for(i = 0; i < opt_num; i++)
-			{
-				guc_options[i] = p;
-				p += (strlen(p) + 1); /* skip option name */
-				p += (strlen(p) + 1); /* skip option value */
-			}
-			/* sort option name using quick sort */
-			qsort( (void *)guc_options, opt_num, sizeof(char *), opt_sort );
-
-			p = sp->startup_packet + sizeof(int);	/* skip protocol version info */
-			for (i = 0; i < opt_num; i++)
-			{
-				tmpopt = guc_options[i];
-				memcpy(p, tmpopt ,strlen(tmpopt) + 1); /* memcpy option name */
-				p += (strlen(tmpopt) + 1);
-				tmpopt += (strlen(tmpopt) + 1);
-				memcpy(p, tmpopt ,strlen(tmpopt) + 1); /* memcpy option value */
-				p += (strlen(tmpopt) + 1);
-			}
-
-			pfree(guc_options);
-			pfree(sp_sort);
-
-			p = sp->startup_packet;
 			p += sizeof(int);	/* skip protocol version info */
 
 			while(*p)
@@ -643,14 +586,6 @@ static StartupPacket *read_startup_packet(POOL_CONNECTION *cp)
 						(errmsg("reading startup packet"),
 							 errdetail("application_name: %s", p)));
 				}
-				else
-				{
-					ereport(DEBUG1,
-							(errmsg("reading startup packet"),
-							 errdetail("guc name: %s value: %s", p, p+strlen(p)+1)));
-					p += (strlen(p) + 1);
-
-				}
 
 				p += (strlen(p) + 1);
 			}
@@ -926,20 +861,19 @@ static StartupPacket *StartupPacketCopy(StartupPacket *sp)
 static POOL_CONNECTION_POOL *connect_backend(StartupPacket *sp, POOL_CONNECTION *frontend)
 {
 	POOL_CONNECTION_POOL *backend;
-	StartupPacket *volatile topmem_sp = NULL;
-	volatile bool	topmem_sp_set = false;
-	int			i;
+	StartupPacket *topmem_sp;
+	int i;
 
 	/* connect to the backend */
 	backend = pool_create_cp();
 	if (backend == NULL)
 	{
-		pool_send_error_message(frontend, sp->major, "XX000", "all backend nodes are down, pgpool requires at least one valid node", "",
-								"repair the backend nodes and restart pgpool", __FILE__, __LINE__);
+		pool_send_error_message(frontend, sp->major, "XX000", "connection cache is full", "",
+								"increase max_pool", __FILE__, __LINE__);
 		ereport(ERROR,
-				(errmsg("unable to connect to backend"),
-				 errdetail("all backend nodes are down, pgpool requires at least one valid node"),
-				 errhint("repair the backend nodes and restart pgpool")));
+			(errmsg("unable to connect to backend"),
+				errdetail("connection cache is full"),
+					errhint("increase the \"max_pool\" configuration value, current max_pool is %d",pool_config->max_pool)));
 	}
 	
 	PG_TRY();
@@ -965,7 +899,6 @@ static POOL_CONNECTION_POOL *connect_backend(StartupPacket *sp, POOL_CONNECTION
 				 * save startup packet info
 				 */
 				CONNECTION_SLOT(backend, i)->sp = topmem_sp;
-				topmem_sp_set = true;
 
 				/* send startup packet */
 				send_startup_packet(CONNECTION_SLOT(backend, i));
@@ -979,17 +912,10 @@ static POOL_CONNECTION_POOL *connect_backend(StartupPacket *sp, POOL_CONNECTION
 	PG_CATCH();
 	{
 		pool_discard_cp(sp->user, sp->database, sp->major);
-		topmem_sp = NULL;
 		PG_RE_THROW();
 	}
 	PG_END_TRY();
 
-	/* At this point, we need to free previously allocated memory for the
-	 * startup packet if no backend is up.
-	 */
-	if (!topmem_sp_set && topmem_sp != NULL)
-		pfree(topmem_sp);
-
 	return backend;
 }
 
@@ -1000,10 +926,10 @@ static RETSIGTYPE die(int sig)
 {
 	int save_errno;
 
-	save_errno = errno;
-
 	POOL_SETMASK(&BlockSig);
 
+	save_errno = errno;
+
 #ifdef NOT_USED
 	ereport(LOG,
 			(errmsg("child process received shutdown request signal %d", sig)));
@@ -1048,9 +974,8 @@ static RETSIGTYPE die(int sig)
 			break;
 	}
 
-	POOL_SETMASK(&UnBlockSig);
-
 	errno = save_errno;
+	POOL_SETMASK(&UnBlockSig);
 }
 
 /*
@@ -1064,12 +989,6 @@ static RETSIGTYPE close_idle_connection(int sig)
 	ConnectionInfo *info;
 	int save_errno = errno;
 
-	/*
-	 * DROP DATABSE is ongoing.
-	 */
-	if (ignore_sigusr1)
-		return;
-
 #ifdef NOT_USED
 	ereport(DEBUG1,
 			(errmsg("close connection request received")));
@@ -1756,7 +1675,7 @@ int select_load_balancing_node(void)
 	/* 
 	 * Check database_redirect_preference_list
 	 */
-	if (STREAM && pool_config->redirect_dbnames)
+	if (SL_MODE && pool_config->redirect_dbnames)
 	{
 		char *database = MASTER_CONNECTION(ses->backend)->sp->database;
 
@@ -1782,7 +1701,7 @@ int select_load_balancing_node(void)
 	/* 
 	 * Check app_name_redirect_preference_list
 	 */
-	if (STREAM && pool_config->redirect_app_names)
+	if (SL_MODE && pool_config->redirect_app_names)
 	{
 		char *app_name = MASTER_CONNECTION(ses->backend)->sp->application_name;
 
@@ -1947,7 +1866,6 @@ wait_for_new_connections(int *fds, struct timeval *timeout, SockAddr *saddr)
 	int fd = 0;
 	int afd;
 	int *walk;
-	int on;
 
 #ifdef ACCEPT_PERFORMANCE
 	struct timeval now1, now2;
@@ -1991,50 +1909,7 @@ wait_for_new_connections(int *fds, struct timeval *timeout, SockAddr *saddr)
 	 */
 	if (SERIALIZE_ACCEPT)
 	{
-		if (pool_config->connection_life_time == 0)
-		{
-			pool_semaphore_lock(ACCEPT_FD_SEM);
-		}
-		else
-		{
-			int sts;
-
-			for (;;)
-			{
-				sts = pool_semaphore_lock_allow_interrupt(ACCEPT_FD_SEM);
-
-				/* Interrupted by alarm */
-				if (sts == -2)
-				{
-					/*
-					 * Check if there are expired connection_life_time.
-					 */
-					if (backend_timer_expired)
-					{
-						/*
-						 * We add 10 seconds to connection_life_time so that there's
-						 * enough margin.
-						 */
-						int	seconds = pool_config->connection_life_time + 10;
-
-						while (seconds-- > 0)
-						{
-							/* check backend timer is expired */
-							if (backend_timer_expired)
-							{
-								pool_backend_timer();
-								backend_timer_expired = 0;
-								break;
-							}
-							sleep(1);
-						}
-					}
-				}
-				else	/* success or other error */
-					break;
-			}
-		}
-
+		pool_semaphore_lock(ACCEPT_FD_SEM);
 		set_ps_display("wait for connection request", false);
 		ereport(DEBUG1,
 			   (errmsg("LOCKING select()")));
@@ -2162,30 +2037,6 @@ wait_for_new_connections(int *fds, struct timeval *timeout, SockAddr *saddr)
 		return RETRY;
 
 	}
-
-	/*
-	 * Set no delay if AF_INET socket. Not sure if this is really necessary
-	 * but PostgreSQL does this.
-	 */
-	if (!FD_ISSET(fds[0], &rmask))	/* fds[0] is UNIX domain socket */
-	{
-		on = 1;
-		if (setsockopt(afd, IPPROTO_TCP, TCP_NODELAY,
-					   (char *) &on,
-					   sizeof(on)) < 0)
-		{
-			ereport(WARNING,
-					(errmsg("wait_for_new_connections: setsockopt failed with error \"%s\"",strerror(errno))));
-			close(afd);
-			return -1;
-		}
-	}
-
-	/*
-	 * Make sure that the socket is non blocking.
-	 */
-	pool_unset_nonblock(afd);
-
 #ifdef ACCEPT_PERFORMANCE
 	gettimeofday(&now2,0);
 	atime += (now2.tv_sec - now1.tv_sec)*1000000 + (now2.tv_usec - now1.tv_usec);
@@ -2258,7 +2109,7 @@ static void validate_backend_connectivity(int front_end_fd)
 		if(front_end_fd > 0)
 		{
 			POOL_CONNECTION *cp;
-			StartupPacket *volatile sp = NULL;
+			StartupPacket *sp;
 
 			/* 
              * we do not want to report socket error, as above errors
@@ -2288,7 +2139,6 @@ static void validate_backend_connectivity(int front_end_fd)
                 FlushErrorState();
 				ereport(FATAL,
 					(errmsg("%s",error_msg),errdetail("%s",error_detail),errhint("%s",error_hint)));
-				pfree(sp);
 			}
 			PG_END_TRY();
 		}
@@ -2376,7 +2226,6 @@ retry_startup:
 	{
 		cancel_request((CancelPacket *)sp->startup_packet);
 		pool_free_startup_packet(sp);
-		connection_count_down();
 		return NULL;
 	}
 
@@ -2392,20 +2241,6 @@ retry_startup:
 		goto retry_startup;
 	}
 
-	/* GSSAPI? */
-	if (sp->major == 1234 && sp->minor == 5680)
-	{
-		ereport(DEBUG1,
-				(errmsg("selecting backend connection"),
-				 errdetail("GSSAPI request from client")));
-
-		/* sorry, Pgpool-II does not support GSSAPI yet */
-		pool_write_and_flush(frontend, "N", 1);
-
-		pool_free_startup_packet(sp);
-		goto retry_startup;
-	}
-
 	frontend->protoVersion = sp->major;
 	frontend->database = pstrdup(sp->database);
 	frontend->username = pstrdup(sp->user);
@@ -2589,161 +2424,3 @@ int pg_frontend_exists(void)
 		return -1;
 	return 0;
 }
-
-static int opt_sort(const void *a, const void *b)
-{
-	return strcmp( *(char **)a, *(char **)b);
-}
-
-/*
- * Returns PostgreSQL version.
- * The returned PgVersion struct is in static memory.
- * Caller must not modify it.
- *
- * Note:
- * Must be called while query context already exists.
- * If there's something goes wrong, this raises FATAL. So never returns to caller.
- *
- */
-PGVersion *
-Pgversion(POOL_CONNECTION_POOL * backend)
-{
-#define VERSION_BUF_SIZE	10
-	static	PGVersion	pgversion;
-	static	POOL_RELCACHE *relcache;
-	char	*result;
-	char	*p;
-	char	buf[VERSION_BUF_SIZE];
-	int		i;
-	int		major;
-	int		minor;
-
-	/*
-	 * First, check local cache. If cache is set, just return it.
-	 */
-	if (pgversion.major != 0)
-	{
-		ereport(DEBUG5,
-				(errmsg("Pgversion: local cache returned")));
-
-		return &pgversion;
-	}
-
-	if (!relcache)
-	{
-		/*
-		 * Create relcache.
-		 */
-		relcache = pool_create_relcache(pool_config->relcache_size, "SELECT version()",
-										string_register_func, string_unregister_func, false);
-		if (relcache == NULL)
-		{
-			ereport(FATAL,
-					(errmsg("Pgversion: unable to create relcache while getting PostgreSQL version.")));
-			return NULL;
-		}
-	}
-
-	/*
-	 * Search relcache.
-	 */
-	result = (char *)pool_search_relcache(relcache, backend, "version");
-	if (result == 0)
-	{
-		ereport(FATAL,
-				(errmsg("Pgversion: unable to search relcache while getting PostgreSQL version.")));
-		return NULL;
-	}
-
-	ereport(DEBUG5,
-			(errmsg("Pgversion: version string: %s", result)));
-
-	/*
-	 * Extract major version number.  We create major version as "version" *
-	 * 10.  For example, for V10, the major version number will be 100, for
-	 * V9.6 it will be 96, and so on.  For alpha or beta version, the version
-	 * string could be something like "12beta1". In this case we assume that
-	 * atoi(3) is smart enough to stop at the first character which is not a
-	 * valid digit (in our case 'b')). So "12beta1" should be converted to 12.
-	 */
-	p = strchr(result, ' ');
-	if (p == NULL)
-	{
-		ereport(FATAL,
-				(errmsg("Pgversion: unable to find the first space in the version string: %s", result)));
-		return NULL;
-	}
-
-	p++;
-	i = 0;
-	while (i < VERSION_BUF_SIZE - 1 && p && *p != '.')
-	{
-		buf[i++] = *p++;
-	}
-	buf[i] = '\0';
-	major = atoi(buf);
-	ereport(DEBUG5,
-			(errmsg("Pgversion: major version: %d", major)));
-
-	/* Assuming PostgreSQL V100 is the final release:-) */
-	if (major < 6 || major > 100)
-	{
-		ereport(FATAL,
-				(errmsg("Pgversion: wrong major version: %d", major)));
-		return NULL;
-	}
-
-	/*
-	 * If major version is 10 or above, we are done to extract major.
-	 * Otherwise extract below decimal point part.
-	 */
-	if (major >= 10)
-	{
-		major *= 10;
-	}
-	else
-	{
-		p++;
-		i = 0;
-		while (i < VERSION_BUF_SIZE -1 && p && *p != '.' && *p != ' ')
-		{
-			buf[i++] = *p++;
-		}
-		buf[i] = '\0';
-		major = major * 10 + atoi(buf);
-		ereport(DEBUG5,
-				(errmsg("Pgversion: major version: %d", major)));
-		pgversion.major = major;
-	}
-
-	/*
-	 * Extract minor version.
-	 */
-	p++;
-	i = 0;
-	while (i < VERSION_BUF_SIZE -1 && p && *p != '.' && *p != ' ')
-	{
-		buf[i++] = *p++;
-	}
-	buf[i] = '\0';
-	minor = atoi(buf);
-	ereport(DEBUG5,
-			(errmsg("Pgversion: minor version: %d", minor)));
-
-	if (minor < 0 || minor > 100)
-	{
-		ereport(FATAL,
-				(errmsg("Pgversion: wrong minor version: %d", minor)));
-		return NULL;
-	}
-
-
-	/*
-	 * Ok, everything looks good. Set the local cache.
-	 */
-	pgversion.major = major;
-	pgversion.minor = minor;
-	strncpy(pgversion.version_string, result, sizeof(pgversion.version_string) - 1);
-
-	return &pgversion;
-}
