@@ -5,7 +5,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2020	PgPool Global Development Group
+ * Copyright (c) 2003-2017	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -60,8 +60,10 @@
 #include "pool_config.h"
 #include "utils/pool_ip.h"
 #include "auth/md5.h"
+#include "auth/pool_hba.h"
 #include "utils/pool_stream.h"
 
+char remote_ps_data[NI_MAXHOST];		/* used for set_ps_display */
 static POOL_CONNECTION_POOL_SLOT	*slots[MAX_NUM_BACKENDS];
 static volatile sig_atomic_t reload_config_request = 0;
 static volatile sig_atomic_t restart_request = 0;
@@ -281,8 +283,6 @@ static void check_replication_time_lag(void)
 
 	for (i=0;i<NUM_BACKENDS;i++)
 	{
-		lsn[i] = 0;
-
 		if (!VALID_BACKEND(i))
 			continue;
 
@@ -329,6 +329,10 @@ static void check_replication_time_lag(void)
 			lsn[i] = text_to_lsn(res->data[0]);
 			free_select_result(res);
 		}
+		else
+		{
+			lsn[i] = 0;
+		}
 	}
 
 	for (i=0;i<NUM_BACKENDS;i++)
