@@ -5,7 +5,7 @@
 * pgpool: a language independent connection pool server for PostgreSQL
 * written by Tatsuo Ishii
 *
-* Copyright (c) 2003-2018	PgPool Global Development Group
+* Copyright (c) 2003-2016	PgPool Global Development Group
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby
@@ -219,7 +219,7 @@ int pool_read(POOL_CONNECTION *cp, void *buf, int len)
 				/* if fail_over_on_backend_error is true, then trigger failover */
 				if (pool_config->fail_over_on_backend_error)
 				{
-					notice_backend_error(cp->db_node_id, true);
+					notice_backend_error(cp->db_node_id, REQ_DETAIL_SWITCHOVER);
 
                     /* If we are in the main process, we will not exit */
 					child_exit(POOL_EXIT_AND_RESTART);
@@ -246,7 +246,7 @@ int pool_read(POOL_CONNECTION *cp, void *buf, int len)
 			cp->socket_state = POOL_SOCKET_EOF;
 			if (cp->isbackend)
 			{
-				if(processType == PT_MAIN)
+				if (processType == PT_MAIN || processType == PT_HEALTH_CHECK)
 					ereport(ERROR,
 						(errmsg("unable to read data from DB node %d",cp->db_node_id),
 							 errdetail("EOF encountered with backend")));
@@ -366,7 +366,7 @@ char *pool_read2(POOL_CONNECTION *cp, int len)
 				/* if fail_over_on_backend_error is true, then trigger failover */
 				if (pool_config->fail_over_on_backend_error)
 				{
-					notice_backend_error(cp->db_node_id, true);
+					notice_backend_error(cp->db_node_id, REQ_DETAIL_SWITCHOVER);
 					child_exit(POOL_EXIT_AND_RESTART);
                     /* we are in main process */
                     ereport(ERROR,
@@ -524,7 +524,7 @@ static int pool_write_flush(POOL_CONNECTION *cp, void *buf, int len)
 	wlen = len;
 
 	ereport(DEBUG1,
-			(errmsg("pool_write_flush: write size: %d", wlen)));
+			(errmsg("pool_write_flush_it: write size: %d", wlen)));
 
 	if (wlen == 0)
 	{
@@ -539,14 +539,14 @@ static int pool_write_flush(POOL_CONNECTION *cp, void *buf, int len)
 
 		if (cp->ssl_active > 0)
 		{
-			sts = pool_ssl_write(cp, buf+offset, wlen);
+		  sts = pool_ssl_write(cp, buf, wlen);
 		}
 		else
 		{
-			sts = write(cp->fd, buf+offset, wlen);
+		  sts = write(cp->fd, buf, wlen);
 		}
 
-		if (sts >= 0)
+		if (sts > 0)
 		{
 			wlen -= sts;
 
@@ -559,7 +559,7 @@ static int pool_write_flush(POOL_CONNECTION *cp, void *buf, int len)
 			else if (wlen < 0)
 			{
 				ereport(WARNING,
-						(errmsg("pool_write_flush: invalid write size %d", sts)));
+						(errmsg("pool_write_flush_it: invalid write size %d", sts)));
 				return -1;
 			}
 
@@ -567,7 +567,7 @@ static int pool_write_flush(POOL_CONNECTION *cp, void *buf, int len)
 			{
 				/* need to write remaining data */
 				ereport(DEBUG1,
-						(errmsg("pool_write_flush: write retry: %d", wlen)));
+						(errmsg("pool_write_flush_it: write retry: %d", wlen)));
 
 				offset += sts;
 				continue;
@@ -633,7 +633,7 @@ int pool_flush_it(POOL_CONNECTION *cp)
 		  sts = write(cp->fd, cp->wbuf + offset, wlen);
 		}
 
-		if (sts >= 0)
+		if (sts > 0)
 		{
 			wlen -= sts;
 
@@ -710,7 +710,7 @@ int pool_flush(POOL_CONNECTION *cp)
 			/* if fail_over_on_backend_error is true, then trigger failover */
 			if (pool_config->fail_over_on_backend_error)
 			{
-				notice_backend_error(cp->db_node_id, true);
+				notice_backend_error(cp->db_node_id, REQ_DETAIL_SWITCHOVER);
 				ereport(LOG,
 					(errmsg("unable to flush data to backend"),
 						 errdetail("do not failover because I am the main process")));
@@ -765,7 +765,7 @@ int pool_flush_noerror(POOL_CONNECTION *cp)
             /* if fail_over_on_backend_erro is true, then trigger failover */
             if (pool_config->fail_over_on_backend_error)
             {
-                notice_backend_error(cp->db_node_id, true);
+                notice_backend_error(cp->db_node_id, REQ_DETAIL_SWITCHOVER);
                 child_exit(POOL_EXIT_AND_RESTART);
 				ereport(LOG,
 					(errmsg("unable to flush data to backend"),
@@ -933,7 +933,7 @@ char *pool_read_string(POOL_CONNECTION *cp, int *len, int line)
 							 errdetail("pg_terminate_backend was called on the backend")));
 				}
 
-				notice_backend_error(cp->db_node_id, true);
+				notice_backend_error(cp->db_node_id, REQ_DETAIL_SWITCHOVER);
 				child_exit(POOL_EXIT_AND_RESTART);
                 ereport(ERROR,
                         (errmsg("unable to read data from frontend"),
